import 'dart:io';

import 'package:shared_preferences/shared_preferences.dart';
import 'package:tencent_calls_engine/tencent_calls_engine.dart';
import 'package:tencent_calls_uikit/src/data/constants.dart';
import 'package:tencent_calls_uikit/src/extensions/calling_bell_feature.dart';
import 'package:tencent_calls_uikit/src/platform/tuicall_kit_platform_interface.dart';
import 'package:tencent_calls_uikit/src/tuicall_data.dart';
import 'package:tencent_calls_uikit/src/tuicall_navigator_observer.dart';
import 'package:tencent_calls_uikit/src/tuicall_state.dart';
import 'package:tencent_calls_uikit/src/utils/event_bus.dart';
import 'package:tencent_calls_uikit/src/utils/permission_request.dart';
import 'package:tencent_calls_uikit/src/utils/preference_utils.dart';
import 'package:tencent_cloud_uikit_core/tencent_cloud_uikit_core.dart';
import 'package:tencent_cloud_chat_sdk/tencent_im_sdk_plugin.dart';

class TUICallEngienManager {
  static final TUICallEngienManager _instance = TUICallEngienManager();

  static TUICallEngienManager get instance => _instance;
  final _im = TencentImSDKPlugin.v2TIMManager;

  TUICallEngienManager() {
    eventBus.register(setStateEventOnCallReceived, (arg) async {
      if (Platform.isAndroid) {
        final permissionResult = await PermissionRequest.checkCallingPermission(
            TUICallState.instance.mediaType);
        if (TUIPermissionResult.granted == permissionResult) {
          TUICallKitNavigatorObserver.getInstance().enterCallingPage();
        } else {
          TUICallEngienManager.instance.reject();
          TUICallKitPlatform.instance.stopRing();
        }
      } else {
        TUICallKitNavigatorObserver.getInstance().enterCallingPage();
      }
    });
  }

  Future<void> initEngine(int sdkAppID, String userId, String userSig) async {
    final result = await TUICallEngine.instance.init(sdkAppID, userId, userSig);

    if (result.code.isEmpty) {
      TUICallEngine.instance.setVideoEncoderParams(VideoEncoderParams(
          resolution: Resolution.resolution_640_360,
          resolutionMode: ResolutionMode.portrait));
      TUICallEngine.instance.setVideoRenderParams(
          userId,
          VideoRenderParams(
              fillMode: FillMode.fill, rotation: Rotation.rotation_0));
      TUICallEngine.instance.setBeautyLevel(6.0);
      _updateLocalSelfUser(userId);
    } else {
      TUIToast.show(content: 'Init Engine Fail');
    }
  }

  Future<TUIResult> call(String userId, TUICallMediaType callMediaType,
      [TUICallParams? params]) async {
    if (userId.isEmpty) {
      print("Call failed, userId is empty");
      return TUIResult(code: "-1", message: "Call failed, userId is empty");
    }
    if (TUICallMediaType.none == callMediaType) {
      print("Call failed, callMediaType is Unknown");
      return TUIResult(
          code: "-1", message: "Call failed, callMediaType is Unknown");
    }
    if (params != null &&
        params.roomId != null &&
        params.roomId!.intRoomId > Constants.roomIdMaxValue) {
      return TUIResult(
          code: "-1",
          message: "Call failed, roomId.intRoomId Max Value is "
              "2147483647");
    }

    if (Platform.isAndroid) {
      final permissionResult =
          await PermissionRequest.checkCallingPermission(callMediaType);
      if (TUIPermissionResult.granted == permissionResult) {
        final callResult = await TUICallEngine.instance
            .call(userId, callMediaType, params ?? TUICallParams());
        if (callResult.code.isEmpty) {
          CallingBellFeature.startRing();
          User user = User();
          user.id = userId;
          user.callRole = TUICallRole.called;
          user.callStatus = TUICallStatus.waiting;
          final imUserInfo = await _im.getUsersInfo(userIDList: [userId]);
          user.nickname = imUserInfo.data?[0].nickName ?? '';
          user.avatar = imUserInfo.data?[0].faceUrl ?? '';
          TUICallState.instance.remoteUserList.add(user);
          TUICallState.instance.mediaType = callMediaType;
          TUICallState.instance.scene = TUICallScene.singleCall;
          TUICallState.instance.selfUser.callRole = TUICallRole.caller;
          TUICallState.instance.selfUser.callStatus = TUICallStatus.waiting;

          TUICallKitPlatform.instance.updateCallStateToNative();
          initAudioPlayDevice();
          TUICallKitNavigatorObserver.getInstance().enterCallingPage();
          return callResult;
        } else {
          return TUIResult(code: "-1", message: "Call Fail, engine call fail");
        }
      } else {
        return TUIResult(code: "-1", message: "Permission result fail");
      }
    } else {
      final callResult = await TUICallEngine.instance
          .call(userId, callMediaType, params ?? TUICallParams());
      if (callResult.code.isEmpty) {
        CallingBellFeature.startRing();
        User user = User();
        user.id = userId;
        user.callRole = TUICallRole.called;
        user.callStatus = TUICallStatus.waiting;
        final imUserInfo = await _im.getUsersInfo(userIDList: [userId]);
        user.nickname = imUserInfo.data?[0].nickName ?? '';
        user.avatar = imUserInfo.data?[0].faceUrl ?? '';
        TUICallState.instance.remoteUserList.add(user);
        TUICallState.instance.mediaType = callMediaType;
        TUICallState.instance.scene = TUICallScene.singleCall;
        TUICallState.instance.selfUser.callRole = TUICallRole.caller;
        TUICallState.instance.selfUser.callStatus = TUICallStatus.waiting;

        TUICallKitPlatform.instance.updateCallStateToNative();
        initAudioPlayDevice();
        TUICallKitNavigatorObserver.getInstance().enterCallingPage();
        return callResult;
      } else {
        return TUIResult(code: "-1", message: "Call Fail, engine call fail");
      }
    }
  }

  Future<TUIResult> groupCall(
      String groupId, List<String> userIdList, TUICallMediaType mediaType,
      [TUICallParams? params]) async {
    if (groupId.isEmpty) {
      print("groupCall failed, groupId is empty");
      return TUIResult(
          code: "-1", message: "groupCall failed, groupId is empty");
    }
    if (TUICallMediaType.none == mediaType) {
      print("groupCall failed, callMediaType is Unknown");
      return TUIResult(
          code: "-1", message: "groupCall failed, callMediaType is Unknown");
    }
    if (userIdList == null || userIdList.length <= 0) {
      print("groupCall failed, userIdList is empty");
      return TUIResult(
          code: "-1",
          message: "groupCall failed, userIdList is "
              "empty");
    }
    if (params != null &&
        params.roomId != null &&
        params.roomId!.intRoomId > Constants.roomIdMaxValue) {
      return TUIResult(
          code: "-1",
          message: "Call failed, roomId.intRoomId Max Value is "
              "2147483647");
    }
    if (userIdList.length >= Constants.groupCallMaxUserCount) {
      TUIToast.show(content: "groupCall failed, exceeding max user number: 9");
      return TUIResult(
          code: "-1",
          message: "groupCall failed, exceeding max user number: 9");
    }
    if (Platform.isAndroid) {
      final permissionResult =
          await PermissionRequest.checkCallingPermission(mediaType);
      if (TUIPermissionResult.granted == permissionResult) {
        final callResult = await TUICallEngine.instance.groupCall(
            groupId, userIdList, mediaType, params ?? TUICallParams());
        if (callResult.code.isEmpty) {
          CallingBellFeature.startRing();
          for (String userId in userIdList) {
            if (userId.isNotEmpty) {
              User user = User();
              user.id = userId;
              user.callRole = TUICallRole.called;
              user.callStatus = TUICallStatus.waiting;
              final imUserInfo = await _im.getUsersInfo(userIDList: [userId]);
              user.nickname = imUserInfo.data?[0].nickName ?? '';
              user.avatar = imUserInfo.data?[0].faceUrl ?? '';
              TUICallState.instance.remoteUserList.add(user);
              TUICallState.instance.calleeList.add(user);
            }
          }

          TUICallState.instance.mediaType = mediaType;
          TUICallState.instance.scene = TUICallScene.groupCall;
          TUICallState.instance.groupId = groupId;

          TUICallState.instance.selfUser.callRole = TUICallRole.caller;
          TUICallState.instance.selfUser.callStatus = TUICallStatus.waiting;
          TUICallState.instance.caller = TUICallState.instance.selfUser;

          TUICallKitPlatform.instance.updateCallStateToNative();
          initAudioPlayDevice();
          TUICallKitNavigatorObserver.getInstance().enterCallingPage();
          return callResult;
        } else {
          return TUIResult(code: "-1", message: "Call Fail, engine call fail");
        }
      } else {
        return TUIResult(code: "-1", message: "Permission result fail");
      }
    } else {
      final callResult = await TUICallEngine.instance
          .groupCall(groupId, userIdList, mediaType, params ?? TUICallParams());
      if (callResult.code.isEmpty) {
        CallingBellFeature.startRing();
        for (String userId in userIdList) {
          if (userId.isNotEmpty) {
            User user = User();
            user.id = userId;
            user.callRole = TUICallRole.called;
            user.callStatus = TUICallStatus.waiting;
            final imUserInfo = await _im.getUsersInfo(userIDList: [userId]);
            user.nickname = imUserInfo.data?[0].nickName ?? '';
            user.avatar = imUserInfo.data?[0].faceUrl ?? '';
            TUICallState.instance.remoteUserList.add(user);
          }
        }

        TUICallState.instance.mediaType = mediaType;
        TUICallState.instance.scene = TUICallScene.groupCall;
        TUICallState.instance.groupId = groupId;

        TUICallState.instance.selfUser.callRole = TUICallRole.caller;
        TUICallState.instance.selfUser.callStatus = TUICallStatus.waiting;

        TUICallKitPlatform.instance.updateCallStateToNative();
        initAudioPlayDevice();
        TUICallKitNavigatorObserver.getInstance().enterCallingPage();
        return callResult;
      } else {
        return TUIResult(code: "-1", message: "Call Fail, engine call fail");
      }
    }
  }

  Future<void> joinInGroupCall(
      TUIRoomId roomId, String groupId, TUICallMediaType mediaType) async {
    if (roomId.intRoomId <= 0 || roomId.intRoomId >= Constants.roomIdMaxValue) {
      print("joinInGroupCall failed, roomId is invalid");
      return;
    }
    if (groupId.isEmpty) {
      print("joinInGroupCall failed, groupId is empty");
      return;
    }
    if (TUICallMediaType.none == mediaType) {
      print("joinInGroupCall failed, mediaType is unknown");
      return;
    }
    if (Platform.isAndroid) {
      final permissionResult =
          await PermissionRequest.checkCallingPermission(mediaType);
      if (TUIPermissionResult.granted == permissionResult) {
        final result = await TUICallEngine.instance
            .joinInGroupCall(roomId, groupId, mediaType);
        if (result.code.isEmpty) {
          TUICallState.instance.groupId = groupId;
          TUICallState.instance.roomId = roomId;
          TUICallState.instance.mediaType = mediaType;
          TUICallState.instance.scene = TUICallScene.groupCall;
          TUICallState.instance.selfUser.callRole = TUICallRole.called;
          TUICallState.instance.selfUser.callStatus = TUICallStatus.accept;

          TUICallKitPlatform.instance.updateCallStateToNative();
          initAudioPlayDevice();
          TUICallKitNavigatorObserver.getInstance().enterCallingPage();
          return;
        } else {
          TUIToast.show(
              content: "joinInGroupCall Fail, engine call "
                  "fail");
          return;
        }
      } else {
        TUIToast.show(content: "Permission result fail");
        return;
      }
    } else {
      final result = await TUICallEngine.instance
          .joinInGroupCall(roomId, groupId, mediaType);
      if (result.code.isEmpty) {
        TUICallState.instance.groupId = groupId;
        TUICallState.instance.roomId = roomId;
        TUICallState.instance.mediaType = mediaType;
        TUICallState.instance.scene = TUICallScene.groupCall;
        TUICallState.instance.selfUser.callRole = TUICallRole.called;
        TUICallState.instance.selfUser.callStatus = TUICallStatus.accept;
        TUICallKitPlatform.instance.updateCallStateToNative();
        initAudioPlayDevice();
        TUICallKitNavigatorObserver.getInstance().enterCallingPage();
        return;
      } else {
        TUIToast.show(content: "joinInGroupCall Fail,engine call fail");
        return;
      }
    }
  }

  Future<TUIResult> accept() async {
    final result = await TUICallEngine.instance.accept();
    if (result.code.isEmpty) {
      TUICallState.instance.selfUser.callStatus = TUICallStatus.accept;
    } else {
      TUICallState.instance.selfUser.callStatus = TUICallStatus.none;
    }

    TUICallKitPlatform.instance.updateCallStateToNative();

    return result;
  }

  Future<TUIResult> reject() async {
    final result = await TUICallEngine.instance.reject();
    TUICallState.instance.selfUser.callStatus = TUICallStatus.none;

    TUICallKitPlatform.instance.updateCallStateToNative();

    return result;
  }

  Future<void> switchCallMediaType(TUICallMediaType mediaType) async {
    TUICallEngine.instance.switchCallMediaType(mediaType);

    TUICallKitPlatform.instance.updateCallStateToNative();
  }

  Future<TUIResult> hangup() async {
    final result = await TUICallEngine.instance.hangup();
    TUICallState.instance.selfUser.callStatus = TUICallStatus.none;

    TUICallKitPlatform.instance.updateCallStateToNative();

    return result;
  }

  Future<TUIResult> openCamera(TUICamera camera, int viewId) async {
    final result = await TUICallEngine.instance.openCamera(camera, viewId);
    if (result.code.isEmpty &&
        TUICallStatus.none != TUICallState.instance.selfUser.callStatus) {
      TUICallState.instance.isCameraOpen = true;
      TUICallState.instance.camera = camera;
      TUICallState.instance.selfUser.videoAvailable = true;
    }

    TUICallKitPlatform.instance.updateCallStateToNative();

    return result;
  }

  Future<void> closeCamera() async {
    TUICallEngine.instance.closeCamera();
    TUICallState.instance.isCameraOpen = false;
    TUICallState.instance.selfUser.videoAvailable = false;

    TUICallKitPlatform.instance.updateCallStateToNative();
  }

  Future<void> switchCamera(TUICamera camera) async {
    TUICallEngine.instance.switchCamera(camera);
    TUICallState.instance.camera = camera;

    TUICallKitPlatform.instance.updateCallStateToNative();
  }

  Future<TUIResult> openMicrophone() async {
    final result = await TUICallEngine.instance.openMicrophone();
    TUICallState.instance.isMicrophoneMute = false;

    TUICallKitPlatform.instance.updateCallStateToNative();

    return result;
  }

  Future<void> closeMicrophone() async {
    TUICallEngine.instance.closeMicrophone();
    TUICallState.instance.isMicrophoneMute = true;

    TUICallKitPlatform.instance.updateCallStateToNative();
  }

  Future<void> selectAudioPlaybackDevice(TUIAudioPlaybackDevice device) async {
    TUICallEngine.instance.selectAudioPlaybackDevice(device);
    TUICallState.instance.audioDevice = device;

    TUICallKitPlatform.instance.updateCallStateToNative();
  }

  Future<void> startRemoteView(String userId, int viewId) async {
    await TUICallEngine.instance.startRemoteView(userId, viewId);

    TUICallKitPlatform.instance.updateCallStateToNative();
  }

  Future<void> stopRemoteView(String userId) async {
    await TUICallEngine.instance.stopRemoteView(userId);

    TUICallKitPlatform.instance.updateCallStateToNative();
  }

  Future<void> enableFloatWindow(bool enable) async {
    TUICallState.instance.enableFloatWindow = enable;
  }

  Future<void> enableMuteMode(bool enable) async {
    TUICallState.instance.enableMuteMode = enable;
    PreferenceUtils.getInstance().saveBool(
        Constants.spKeyEnableMuteMode, TUICallState.instance.enableMuteMode);
  }

  Future<TUIResult> setSelfInfo(String nickname, String avatar) async {
    final result = await TUICallEngine.instance.setSelfInfo(nickname, avatar);
    return result;
  }

  Future<void> inviteUser(List<String> userIdList) async {
    TUICallParams params = TUICallParams();
    params.offlinePushInfo = TUIOfflinePushInfo();
    await TUICallEngine.instance.iniviteUser(
        userIdList,
        params,
        TUIValueCallback(
            onSuccess: (userIds) async {
              if (userIds != null && userIds is List && userIds.length > 0) {
                List<String> list = [];
                for (Object userId in userIds) {
                  if (userId is String) {
                    list.add(userId);
                  }
                }
                final imUserList = await _im.getUsersInfo(userIDList: list);
                if (imUserList != null && imUserList.data != null) {
                  imUserList.data?.forEach((item) {
                    User user = User();
                    user.id = item.userID!;
                    user.nickname = item.nickName ?? item.userID ?? "";
                    user.avatar = item.faceUrl ?? "";
                    user.callStatus = TUICallStatus.waiting;
                    TUICallState.instance.remoteUserList.add(user);
                  });
                  eventBus.notify(setStateEvent);
                }
              }
            },
            onError: (code, message) {}));
  }

  void _updateLocalSelfUser(String userId) async {
    TUICallState.instance.selfUser.id = userId;
    final imInfo = await _im.getUsersInfo(userIDList: [userId]);
    TUICallState.instance.selfUser.nickname = imInfo.data?[0].nickName ?? '';
    TUICallState.instance.selfUser.avatar = imInfo.data?[0].faceUrl ?? '';
  }

  void initAudioPlayDevice() {
    if (TUICallMediaType.audio == TUICallState.instance.mediaType &&
        TUICallScene.singleCall == TUICallState.instance.scene) {
      TUICallState.instance.audioDevice = TUIAudioPlaybackDevice.earpiece;
    } else {
      TUICallState.instance.audioDevice = TUIAudioPlaybackDevice.speakerphone;
    }
    TUICallEngienManager.instance
        .selectAudioPlaybackDevice(TUICallState.instance.audioDevice);
  }

  void adaptiveComponentReport() async {
    await TUICallEngine.instance.callExperimentalAPI({
      "api": "setFramework",
      "params": {
        "framework": 7,
        "component": 14,
        "language": 9,
      }
    });
  }
}
