import 'dart:async';
import 'dart:io';
import 'package:flutter/material.dart';
import 'package:tencent_calls_engine/tencent_calls_engine.dart';
import 'package:tencent_calls_uikit/src/tuicall_engine_manager.dart';
import 'package:tencent_calls_uikit/src/tuicall_state.dart';
import 'package:tencent_calls_uikit/src/ui/widget/common/timing_widget.dart';
import 'package:tencent_calls_uikit/src/tuicall_navigator_observer.dart';
import 'package:tencent_calls_uikit/src/platform/tuicall_kit_platform_interface.dart';
import 'package:tencent_calls_uikit/src/ui/widget/singlecall/single_function_widget.dart';
import 'package:tencent_calls_uikit/src/utils/event_bus.dart';
import 'package:tencent_calls_uikit/src/i18n/i18n_utils.dart';
import 'package:tencent_calls_uikit/src/data/constants.dart';

class SingleCallWidget extends StatefulWidget {
  final Function close;

  const SingleCallWidget({
    Key? key,
    required this.close,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _SingleCallWidgetState();
}

class _SingleCallWidgetState extends State<SingleCallWidget> {
  double _smallViewTop = 128;
  double _smallViewRight = 20;

  Timer? _timer;
  late EventCallback callBeginCallBack = (arg) {
    _timer = Timer.periodic(const Duration(milliseconds: 100), (timer) {
      _changeVideoView();
      timer.cancel();
    });
  };

  @override
  void initState() {
    super.initState();

    if (TUICallState.instance.mediaType == TUICallMediaType.video) {
      TUICallState.instance.isCameraOpen = true;
    }
    TUICallState.instance.isChangedBigSmallVideo = false;

    eventBus.register(setStateEventOnCallBegin, callBeginCallBack);
  }

  @override
  dispose() {
    super.dispose();
    TUICallState.instance.isChangedBigSmallVideo = false;
    eventBus.unregister(setStateEventOnCallBegin, callBeginCallBack);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: WillPopScope(
        onWillPop: () async {
          return true;
        },
        child: Stack(
          alignment: Alignment.topLeft,
          fit: StackFit.expand,
          children: [
            Container(
              color: _getBackgroundColor(),
              child: _getBigVideoWidget(),
            ),
            _getSmallVideoWidget(),
            _getFloatingWindowBtnWidget(),
            _getUserInfoWidget(),
            _getFunctionButtonWidget(),
          ],
        ),
      ),
    );
  }

  _getFloatingWindowBtnWidget() {
    return TUICallState.instance.enableFloatWindow ? Positioned(
      left: 24,
      top: 64,
      width: 30,
      height: 30,
      child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            InkWell(
              child: Image.asset(
                TUICallState.instance.mediaType == TUICallMediaType.audio
                    ? 'assets/images/floating_button_black.png'
                    : 'assets/images/floating_button_white.png',
                package: 'tencent_calls_uikit',
              ),
              onTap: () {
                _openFlaotWindow();
              },
            )
          ]),
    ) : const SizedBox();
  }

  _getUserInfoWidget() {
    var showName = '';
    var avatar = '';
    if (TUICallState.instance.remoteUserList.isNotEmpty) {
      showName = TUICallState.instance.remoteUserList[0].nickname ?? '';
      avatar = TUICallState.instance.remoteUserList[0].avatar ?? Constants.defaultAvatar;
    }

    bool isWaiting =
        TUICallState.instance.selfUser.callStatus == TUICallStatus.waiting
            ? true
            : false;

    var videoUserInfoWidget = Positioned(
        top: _smallViewTop - 60,
        right: _smallViewRight,
        child: Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: isWaiting
              ? [
                  Column(
                    children: [
                      Text(
                        showName,
                        style: TextStyle(
                          fontSize: 24,
                          color: _getTextColor(),
                          fontWeight: FontWeight.bold,
                        ),
                      ),
                      const SizedBox(
                        height: 20,
                      ),
                      Text(
                        TUICallState.instance.selfUser.callRole ==
                                TUICallRole.caller
                            ? CallKit_t("正在等待对方接受邀请……")
                            : CallKit_t("邀请您进行通话……"),
                        style: TextStyle(fontSize: 12, color: _getTextColor()),
                      )
                    ],
                  ),
                  const SizedBox(
                    width: 30,
                  ),
                  Container(
                      height: 110,
                      width: 110,
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(8),
                        image: DecorationImage(
                          image: NetworkImage(avatar),
                          fit: BoxFit.cover,
                        ),
                      ),
                      child: Container()),
                ]
              : [],
        ));

    var audioUserInfoWidget = Positioned(
        top: _smallViewTop,
        width: MediaQuery.of(context).size.width,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Container(
                height: 110,
                width: 110,
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(8),
                  image: DecorationImage(
                    image: NetworkImage(avatar),
                    fit: BoxFit.cover,
                  ),
                ),
                child: Container()),
            const SizedBox(height: 20),
            Text(
              showName,
              style: TextStyle(
                fontSize: 24,
                color: _getTextColor(),
                fontWeight: FontWeight.bold,
              ),
            ),
            const SizedBox(height: 10),
            isWaiting
                ? Text(
                    TUICallState.instance.selfUser.callRole ==
                            TUICallRole.caller
                        ? CallKit_t("正在等待对方接受邀请……")
                        : CallKit_t("邀请您进行通话……"),
                    style: TextStyle(fontSize: 12, color: _getTextColor()),
                  )
                : const SizedBox()
          ],
        ));

    return TUICallState.instance.mediaType == TUICallMediaType.audio
        ? audioUserInfoWidget
        : videoUserInfoWidget;
  }

  _getFunctionButtonWidget() {
    var timerText = Container(
      margin: const EdgeInsets.only(bottom: 10),
      child: const TimingWidget(),
    );

    var buttomWidget = Positioned(
      left: 0,
      bottom: 50,
      width: MediaQuery.of(context).size.width,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          TUICallState.instance.selfUser.callStatus == TUICallStatus.accept
              ? timerText
              : const SizedBox(),
          SingleFunctionWidget.buildFunctionWidget(widget.close)
        ],
      ),
    );
    return buttomWidget;
  }

  // 占满全屏幕的大窗
  Widget _getBigVideoWidget() {

    var remoteAvatar = '';
    var remoteVideoAvailable = false;
    if (TUICallState.instance.remoteUserList.isNotEmpty) {
      remoteAvatar = TUICallState.instance.remoteUserList[0].avatar ?? Constants.defaultAvatar;
      remoteVideoAvailable = TUICallState.instance.remoteUserList[0].videoAvailable;
    }
    var selfAvatar = TUICallState.instance.selfUser.avatar ?? Constants.defaultAvatar;
    var isCamerOpen = TUICallState.instance.isCameraOpen;

    if (TUICallState.instance.mediaType == TUICallMediaType.audio) {
      return const SizedBox();
    }

    bool isLocalViewBig = true;
    if (TUICallState.instance.selfUser.callStatus == TUICallStatus.waiting) {
      isLocalViewBig = true;
    } else {
      if (TUICallState.instance.isChangedBigSmallVideo) {
        isLocalViewBig = false;
      } else {
        isLocalViewBig = true;
      }
    }


    return TUICallState.instance.mediaType == TUICallMediaType.video ?
        Container(
          color: Colors.black54,
          child: Stack(
            children: [

              TUICallState.instance.selfUser.callStatus == TUICallStatus.accept ? Column(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Container(
                          height: 80,
                          width: 80,
                          decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(0),
                            image: DecorationImage(
                              image: NetworkImage(
                                  isLocalViewBig ? selfAvatar : remoteAvatar
                              ),
                              fit: BoxFit.cover,
                            ),
                          )
                      ),
                    ],
                  ),
                ],
              ) : Container() ,

              AnimatedOpacity(
                duration: const Duration(milliseconds: 100),
                opacity: isLocalViewBig ? _getOpacityByVis(isCamerOpen) : _getOpacityByVis(remoteVideoAvailable),
                child: TUIVideoView(
                    key: const ValueKey("_bigVideoViewId"),
                    onPlatformViewCreated: (viewId) {
                      TUICallState.instance.selfUser.viewID = viewId;
                      TUICallEngienManager.instance
                          .openCamera(TUICallState.instance.camera, viewId);
                      setState(() {});
                    }
                )
              )
            ],
          ),
        ) : Container();
  }

  Widget _getSmallVideoWidget() {
    if (TUICallState.instance.mediaType == TUICallMediaType.audio) {
      return const SizedBox();
    }
    bool isRemoteViewSmall = true;

    if (TUICallState.instance.selfUser.callStatus == TUICallStatus.accept) {
      if (TUICallState.instance.isChangedBigSmallVideo) {
        isRemoteViewSmall = false;
      } else {
        isRemoteViewSmall = true;
      }
    }

    var remoteAvatar = '';
    var remoteVideoAvailable = false;
    if (TUICallState.instance.remoteUserList.isNotEmpty) {
      remoteAvatar = TUICallState.instance.remoteUserList[0].avatar ?? Constants.defaultAvatar;
      remoteVideoAvailable = TUICallState.instance.remoteUserList[0].videoAvailable;
    }
    var selfAvatar = TUICallState.instance.selfUser.avatar ?? Constants.defaultAvatar;
    var isCamerOpen = TUICallState.instance.isCameraOpen;

    var smallVideoWidget = TUICallState.instance.selfUser.callStatus == TUICallStatus.accept
        ? Container(
      height: 216,
      width: 110,
      color: Colors.black54,
      child: Stack(
                alignment: AlignmentDirectional.center,
                children: [
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      const SizedBox(width: 15),
                      Container(
                          height: 80,
                          width: 80,
                          decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(0),
                            image: DecorationImage(
                              image: NetworkImage(
                                  isRemoteViewSmall ? remoteAvatar : selfAvatar
                              ),
                              fit: BoxFit.cover,
                            ),
                          )
                      ),
                      const SizedBox(width: 15),
                    ],
                  ),

                  AnimatedOpacity(
                      duration: const Duration(milliseconds: 100),
                      opacity: isRemoteViewSmall ? _getOpacityByVis(remoteVideoAvailable) : _getOpacityByVis(isCamerOpen),
                      child: TUIVideoView(
                          key: const ValueKey("_smallVideoViewId"),
                          onPlatformViewCreated: (viewId) {
                            TUICallState.instance.remoteUserList[0].viewID = viewId;
                            TUICallEngienManager.instance.startRemoteView(
                                TUICallState.instance.remoteUserList[0].id, viewId);
                            setState(() {});
                          })
                  )
                ],
          )
    ) : Container();

    return Positioned(
        top: _smallViewTop - 60,
        right: _smallViewRight,
        child: GestureDetector(
          onTap: () {
              _changeVideoView();
          },
          onPanUpdate: (DragUpdateDetails e) {
            if (TUICallState.instance.mediaType == TUICallMediaType.video) {
              _smallViewRight -= e.delta.dx;
              _smallViewTop += e.delta.dy;
              if (_smallViewTop < 100) {
                _smallViewTop = 100;
              }
              if (_smallViewTop > MediaQuery.of(context).size.height - 216) {
                _smallViewTop = MediaQuery.of(context).size.height - 216;
              }
              if (_smallViewRight < 0) {
                _smallViewRight = 0;
              }
              if (_smallViewRight > MediaQuery.of(context).size.width - 110) {
                _smallViewRight = MediaQuery.of(context).size.width - 110;
              }
              setState(() {

              });
            }
          },
          child: SizedBox(
            width: 110,
            child: smallVideoWidget,
          ),
        ));
  }

  _changeVideoView() {
    if (TUICallState.instance.mediaType == TUICallMediaType.audio ||
        TUICallState.instance.selfUser.callStatus == TUICallStatus.waiting) {
      return;
    }

    final selfViewId = TUICallState.instance.selfUser.viewID;
    final selfViewCamera = TUICallState.instance.camera;
    final remoteViewId = TUICallState.instance.remoteUserList[0].viewID;
    final remoteId = TUICallState.instance.remoteUserList[0].id;

    TUICallState.instance.isChangedBigSmallVideo =
        !TUICallState.instance.isChangedBigSmallVideo;

    if (TUICallState.instance.isChangedBigSmallVideo) {
      if(TUICallState.instance.isCameraOpen) {
        TUICallEngienManager.instance.openCamera(selfViewCamera, remoteViewId);
      }
      TUICallEngienManager.instance.startRemoteView(remoteId, selfViewId);
    } else {
      if(TUICallState.instance.isCameraOpen) {
        TUICallEngienManager.instance.openCamera(selfViewCamera, selfViewId);
      }
      TUICallEngienManager.instance.startRemoteView(remoteId, remoteViewId);
    }
    setState(() {});
  }

  double _getOpacityByVis(bool vis) {
    return vis ? 1.0 : 0;
  }

  _openFlaotWindow() async {
    if (Platform.isAndroid) {
      bool result = await TUICallKitPlatform.instance.hasFloatPermission();
      if (!result) {
        return;
      }
    }
    TUICallKitNavigatorObserver.getInstance().exitCallingPage();
    TUICallKitPlatform.instance.startFloatWindow();
  }

  _getBackgroundColor() {
    return TUICallState.instance.mediaType == TUICallMediaType.audio
        ? const Color(0xFFF2F2F2)
        : const Color(0xFF444444);
  }

  _getTextColor() {
    return TUICallState.instance.mediaType == TUICallMediaType.audio
        ? Colors.black
        : Colors.white;
  }
}
